
var ADDED_ITEMS = [];
var DELETED_ITEMS = [];
var EDITED_ITEMS = [];

function initializeEditor(file, key) {
    $("#editor .edit textarea").val("");
    var translations = LOCALIZATIONS[file]["content"][key];
    for(language in translations) {
        text = translations[language];
        $input_box = $("#editor dd.lang-" + language);
        if($input_box.length) {
            $input_box.find("textarea").val(text);
        }
        else {
            alert("WARNING!\nLanguage missed in settings: \"" + language + "\"!");
        }
    }
    $("#editor dd").show();
    
    $("#editor .edit textarea").unbind("change").change(function() {
        language = $(this).closest("dd").attr("class").split(" lang-").pop().split(" ").shift();
        translations[language] = $(this).val();
        LOCALIZATIONS[file]["content"][key] = translations;
        LOCALIZATIONS[file]["status"] = "edited";
        $("dl dd.current").removeClass("loaded").addClass("edited");
        var id = file + "/" + key;
        EDITED_ITEMS[EDITED_ITEMS.length] = id;
    })
}

function initializeKeys(file) {
    $("#keys_list dl dd").remove();
    $("#editor dl dd").hide();
    
    var content = LOCALIZATIONS[file]["content"];
    for(key in content) {
        id = file + "/" + key;
        if(ADDED_ITEMS.indexOf(id) > -1) {
            status = "new";
        }
        else if(DELETED_ITEMS.indexOf(id) > -1) {
            status = "deleted";
        }
        else if(EDITED_ITEMS.indexOf(id) > -1) {
            status = "edited";
        }
        else {
            status = "loaded";
        };
        $("#keys_list dl").append("<dd class='item " + status + "'><span class='marker'>&bull;</span><span class='name'>" + key + "</span><button class='move'>&#x21c4;</button><button class='rename'>&rdquo;</button><button class='delete'>x</button></dd>");
    }
    $("#keys_list dl").append("<dd class='add'><button class='add'>+</button></dd>");
    
    $("#keys_list dd.item").click(function() {
        if($(this).attr("class").indexOf("deleted") < 0) {
            $("#keys_list dd.current").removeClass("current");
            $(this).addClass("current");
            var key = $(this).find(".name").text();
            initializeEditor(file, key);
        }
        else {
            alert("Can't edit deleted item!");
        }
    });
    
    $("#keys_list dd.add").click(function() {
        $(this).blur();
        var key = prompt("New keyword").toLowerCase();
        if(key == "") {
            alert("Keyword can't be empty!");
        }
        else if(Object.keys(content).indexOf(key) > -1) {
            alert("Keyword already exists!");
        }
        else {
            content[key] = {};
            $("#editor .edit").each(function() {
                language = $(this).attr("data-lang");
                content[key][language] = "";
            });
            LOCALIZATIONS[file]["content"] = objectSort(content);
            var id = file + "/" + key;
            ADDED_ITEMS[ADDED_ITEMS.length] = id;
            initializeKeys(file);
            setTimeout(function() {
                $("#keys_list dd.item").each(function() {
                    if($(this).find(".name").text() == key) {
                        $(this).click();
                    }
                })
            }, 100);
        }
    });
    
    $("#keys_list dd .delete").click(function() {
        $(this).blur();
        var $dd = $(this).closest("dd");
        var key = $dd.find(".name").text();
        var id = file + "/" + key;
        // Delete NEW
        if(ADDED_ITEMS.indexOf(id) > -1) {
            if(confirm("Delete: \"" + key + "\"\nAre you sure?")) {
                ADDED_ITEMS = deleteValue(ADDED_ITEMS, id);
                delete content[key];
                LOCALIZATIONS[file]["content"] = content;
                initializeKeys(file);
            }
        }
        // Delete existing
        else {
            if(confirm("Delete: \"" + key + "\"\nAre you sure?")) {
                DELETED_ITEMS[DELETED_ITEMS.length] = id;
                initializeKeys(file);
            }
        };
    });
    
    $("#keys_list dd .rename").click(function() {
        $(this).blur();
        var $dd = $(this).closest("dd");
        var key = $dd.find(".name").text();
        var content = LOCALIZATIONS[file]["content"];
        var rename = prompt("Rename to");
        if(rename == "") {
            alert("Keyword can't be empty");
        }
        else if(Object.keys(content).indexOf(rename) > -1) {
            alert("Keyword already exists!");
        }
        else if(typeof(rename) == "string") {
            data = content[key];
            content[rename] = data;
            delete content[key];
            LOCALIZATIONS[file]["content"] = objectSort(content);
            LOCALIZATIONS[file]["status"] = "edited";
            EDITED_ITEMS[EDITED_ITEMS.length] = file + "/" + rename;

            var current_name = $("#keys_list dd.current .name").text();
            $("#files_list dd.current").removeClass("loaded").addClass("edited");
            initializeKeys(file);
            if(typeof(current_name) == "string" && current_name != "") {
                if(current_name == key) { current_name = rename; };
                setTimeout(function() {
                    $("#keys_list dd.item").each(function() {
                        var item_name = $(this).find(".name").text();
                        if(item_name == current_name) {
                            $(this).click();
                        }
                    })
                }, 100);
            }
        }
    });
    
    $("#keys_list dd .move").click(function() {
        $(this).blur();
        $dd = $(this).closest("dd");
        $dd.css("background-color", "#111");
        
        $dd.draggable({
            revert: true,
            revertDuration: 100,
            cursor: "grabbing",
            zIndex: 100,
            stop: function() {
                $key = $(this);
                setTimeout(function() {
                    $key.attr("style", "");
                    disableDraggable();
                }, 200);
            }
        });

        $("#files_list dd.item").droppable({
            hoverClass: "drop-hover",
            drop: function(event, ui) {
                var to_file = $(this).find(".name").text();
                var key = $(ui.draggable).find(".name").text();
                
                $(ui.draggable).remove();
                moveItem(key, file, to_file);
                
                $("#files_list dd.current").removeClass("loaded").addClass("edited");
                $(this).removeClass("loaded").addClass("edited");
            }
        });
        
        $("#files_list dd.current, #files_list dd.deleted").droppable("destroy");
    });
}

function moveItem(key, from_file, to_file) {
    var from_content = LOCALIZATIONS[from_file]["content"];
    var to_content = LOCALIZATIONS[to_file]["content"];
    to_content[key] = from_content[key];
    delete from_content[key];
    LOCALIZATIONS[from_file]["content"] = objectSort(from_content);
    LOCALIZATIONS[from_file]["status"] = "edited";
    LOCALIZATIONS[to_file]["content"] = objectSort(to_content);
    LOCALIZATIONS[to_file]["status"] = "edited";
    ADDED_ITEMS[ADDED_ITEMS.length] = to_file + "/" + key;
    if(ADDED_ITEMS.indexOf(from_file + "/" + key) > -1) {
        deleteValue(ADDED_ITEMS, from_file + "/" + key);
    }
    $("#editor dl dd").hide();
}

function disableDraggable() {
    $(".ui-draggable").draggable("destroy");
    $(".ui-droppable").droppable("destroy");
    ("#files_list dd.current").attr("style", "");
}

function initializeList() {
    $("#files_list dl dd").remove();
    $("#keys_list dl dd").remove();
    $("#editor dl dd").hide();
    
    for(file in LOCALIZATIONS) {
        var status = LOCALIZATIONS[file]["status"];
        $("#files_list dl").append("<dd class='item " + status + "'><span class='marker'>&bull;</span><span class='name'>" + file + "</span><button class='rename'>&rdquo;</button><button class='delete'>x</button></dd>");
    }
    $("#files_list dl").append("<dd class='add'><button class='add'>+</button></dd>");
    
    $("#files_list dd.item").click(function() {
        if($(this).attr("class").indexOf("deleted") < 0) {
            $("#files_list dd.current").removeClass("current");
            $(this).addClass("current");
            var file = $(this).find(".name").text();
            initializeKeys(file);
        }
        else {
            alert("Can't edit deleted item!");
        }
    });
    
    $("#files_list dd.add").click(function() {
        $(this).blur();
        var file = prompt("New filename").toLowerCase();
        if(file == "") {
            alert("Filename can't be empty!");
        }
        else if(Object.keys(LOCALIZATIONS).indexOf(file) > -1) {
            alert("Filename already exists!");
        }
        else {
            LOCALIZATIONS[file] = { "status": "new", "content": {} };
            LOCALIZATIONS = objectSort(LOCALIZATIONS);
            initializeList();
        }
    });
    
    $("#files_list dd .delete").click(function() {
        $(this).blur();
        var $dd = $(this).closest("dd");
        var file = $dd.find(".name").text();
        // Delete NEW
        if($dd.attr("class").indexOf("new") > -1) {
            if(confirm("Delete: \"" + file + "\"\nAre you sure?")) {
                delete LOCALIZATIONS[file];
                initializeList();
            }
        }
        // Delete existing
        else {
            if(confirm("Delete: \"" + file + "\"\nAre you sure?")) {
                LOCALIZATIONS[file]["status"] = "deleted";
                initializeList();
                DELETED_ITEMS[DELETED_ITEMS.length] = file;
            }
        };
    });
    
    $("#files_list dd .rename").click(function() {
        $(this).blur();
        var $dd = $(this).closest("dd");
        var file = $dd.find(".name").text();
        var rename = prompt("Rename to");
        if(rename == "") {
            alert("Filename can't be empty");
        }
        else if(Object.keys(LOCALIZATIONS).indexOf(rename) > -1) {
            alert("Filename already exists!");
        }
        else if(typeof(rename) == "string"){
            var data = LOCALIZATIONS[file];
            LOCALIZATIONS[rename] = data;
            data["status"] = "edited";
            delete LOCALIZATIONS[file];
            LOCALIZATIONS = objectSort(LOCALIZATIONS);
            DELETED_ITEMS[DELETED_ITEMS.length] = file;
            if(DELETED_ITEMS.indexOf(rename) > -1) {
                deleteValue(DELETED_ITEMS, rename);
            };
            var current_file = $("#files_list dd.current .name").text();
            var current_key = $("#keys_list dd.current .name").text();
            initializeList();
            
            if(typeof(current_file) == "string" && current_file != "") {
                if(file == current_file) { current_file = rename; };
                setTimeout(function() {
                    $("#files_list dd").each(function() {
                        var item_name = $(this).find(".name").text();
                        if(item_name == current_file) {
                            $(this).click();
                            
                            if(typeof(current_key) == "string" && current_key != "") {
                                setTimeout(function() {
                                    $("#keys_list dd").each(function() {
                                        var item_name = $(this).find(".name").text();
                                        if(item_name == current_key) { $(this).click(); };
                                    })
                                }, 100)
                            };
                            
                        }
                    })
                }, 100)
            };
        }
    });
};

function deleteDeleted(obj) {
    for(i in DELETED_ITEMS) {
        var id = DELETED_ITEMS[i].split("/");
        var file = id.shift();
        if(id.length) {
            var key = id.pop();
            var content = obj[file]["content"];
            delete content[key];
            obj[file]["content"] = content;
        }
        else {
            delete obj[file];
        }
    }
    return obj;
};

function statusOff(obj) {
    var output = {};
    for(file in obj) {
        output[file] = obj[file]["content"];
    }
    return output;
};

function detectChanges() {
    var loaded = statusOff( JSON.parse( $("#output .loaded").val() ) )
    var edited = statusOff( deleteDeleted(LOCALIZATIONS) );

    var loaded_json = JSON.stringify(loaded);
    var edited_json = JSON.stringify(edited);

    if(loaded_json != edited_json) { return true; } else { return false; };
};

function joinLanguages(obj) {
    var output = {};
    for(language in obj) {
        var translations = obj[language];
        for(file in translations) {
            var content = translations[file];
            for(key in content) {
                text = content[key];
                if(typeof(text) != "string") { text = ""; }
                if(Object.keys(output).indexOf(file) < 0) { output[file] = { "status": "loaded", "content": {} }; }
                if(Object.keys(output[file]["content"]).indexOf(key) < 0) { output[file]["content"][key] = {}; };
                output[file]["content"][key][language] = text;
            }
        }
    }
    return output;
}

function splitLanguages(obj) {
    var output = {};
    for(file in obj) {
        var content = obj[file]["content"];
        for(key in content) {
            var tranlations = content[key];
            for(language in tranlations) {
                var text = tranlations[language];
                if(typeof(text) != "string") { text = ""; }
                if(Object.keys(output).indexOf(language) < 0) { output[language] = {}; }
                if(Object.keys(output[language]).indexOf(file) < 0) { output[language][file] = {}; }
                output[language][file][key] = text;
            }
        }
    }
    return output;
};

function searchItem() {
    $("#found_box").remove();
    var found = [];
    var find = $("#search input").val();
    if(find.length < 3) {
        alert("At least 3 characters needed");
    }
    else {
        find = find.toLowerCase();
        var find_regex = new RegExp(find,"gi");
        for(file in LOCALIZATIONS) {
            var content = LOCALIZATIONS[file]["content"];
            for(key in content) {
                if(key.indexOf(find) > -1) {
                    id = file + "/" + key;
                    text = key.replace(find_regex, "<span class='found'>" + find.toUpperCase() + "</span>");
                    found[found.length] = { "id": id, "text": text };
                }
                
                var translation = content[key];
                for(language in translation) {
                    var text = translation[language];
                    var test = text.toLowerCase();
                    if(test.indexOf(find) > -1) {
                        id = file + "/" + key;
                        text = text.replace(find_regex, "<span class='found'>" + find.toUpperCase() + "</span>");
                        found[found.length] = { "id": id, "text": text };
                    }
                }
                
            }
        }
        if(found.length) {
            var html = "<div id='found_box'><h3>Found " + found.length + " items</h3><hr>";
            html = html + "<table>";
            for(i in found) {
                var item = found[i];
                html = html + "<tr><td class='id' data-id='" + item["id"] + "'>" + item["id"].replace("/", " / ") + "</td><td class='text'>" + item["text"] + "</td></tr>";
            }
            html = html + "</table>";
            html = html + "<button class='minimize'>&mdash;</button>";
            html = html + "<button class='close'>x</button>";
            html = html + "</div>";
        }
        else {
            var html = "<div id='found_box'><h3>Nothing found</h3>";
            html = html + "<button class='close'>x</button>";
            html = html + "</div>";
        };
        $("body").append(html);
        $("#found_box").slideDown(100);
        
        $("#found_box button.close").click(function() {
            $(this).blur();
            $("#found_box").slideUp(100);
            $("#search input").val("");
        });       
        
        $("#found_box button.minimize").click(function() {
            $(this).blur();
            if($("#found_box table").css("display") == "none") {
                $("#found_box table").slideDown(100);
            }
            else {
                $("#found_box table").slideUp(100);
            }
        });

        $("#found_box td.id").click(function() {
            var key = $(this).attr("data-id");
            if(typeof(key) == "string") {
                data = key.split("/");
                file = data.shift();
                key = data.pop();
                $("#files_list dd.item").each(function() {
                    if($(this).find(".name").text() == file) {
                        $(this).click();
                    }
                })
                setTimeout(function() {
                    $("#keys_list dd.item").each(function() {
                        if($(this).find(".name").text() == key) {
                            $(this).click();
                        }
                    })
                }, 100)
            }
        });

    }
}

$(document).ready(function() {
    LOCALIZATIONS = joinLanguages(LOCALIZATIONS);
    LOCALIZATIONS = objectSort(LOCALIZATIONS);
    $("#output .loaded").val(JSON.stringify(LOCALIZATIONS));
    initializeList();
    
    $("#search button").click(function() { searchItem(); });
    $("#search input").keypress(function (e) {
      if (e.which == 13) {
        searchItem();
        return false;
      }
    });
    
    
    $("header button.cancel").click(function() {
        $(this).blur();
        var loaded_json = $("#output .loaded").val();
        var edited = deleteDeleted(LOCALIZATIONS);
        var edited_json = JSON.stringify(edited);
        if(detectChanges()) {
            if(confirm("Discart all changes.\nAre you sure?")) {
                LOCALIZATIONS = JSON.parse( $("#output .loaded").val() );
                initializeList();
                $("#output textarea.save_file").remove();
                ADDED_ITEMS = [];
                DELETED_ITEMS = [];
                EDITED_ITEMS = [];
            }
        }
        else {
            alert("No changes to discart.");
        }
    });
    
    $("header button.save").click(function() {
        $(this).blur();
        if(detectChanges()) {
            if(confirm("Save all files?")) {
                var languages_data = splitLanguages(deleteDeleted(LOCALIZATIONS));
                for(language in languages_data) {
                    $("#output").append("<textarea class='save json-" + language + "' name='json|" + language + "'></textarea>");
                    $("#output textarea.json-" + language).val( JSON.stringify( languages_data[language] ) );
                }
                $("#output").submit();
            }
        }
        else {
            alert("No changes to save.");
        }
    });
    
    $("#loader").fadeOut(250);
    
})